#import "DUView.h"

#include "fileops.h"

@implementation DUView


- (id)initWithFrame:(NSRect)frame {
  [super initWithFrame:frame];
  
  x_scale=150;
  y_scale=1.0/1024;
 
  data_color=[[NSColor colorWithCalibratedRed:.7 green:.7 blue:1 alpha:1] retain];
  rsrc_color=[[NSColor colorWithCalibratedRed:.7 green:1 blue:.7 alpha:1] retain];
  padding_color=[[NSColor colorWithCalibratedRed:.7 green:.7 blue:.7 alpha:1] retain];
  
  return self;
}



- (IBAction)setYZoom:(id)sender {
  double fact=[sender doubleValue];
  y_scale=1.0/(exp(fact*log(1024)));
  [self handleSizeChange];
  [self setNeedsDisplay: YES];
}

- (IBAction)open:(id)sender {
  NSOpenPanel *oPanel = [NSOpenPanel openPanel];

  [oPanel setCanChooseFiles:NO];
  [oPanel setCanChooseDirectories:YES];
  [oPanel setAllowsMultipleSelection:NO];
   
  if([oPanel runModalForDirectory:NSHomeDirectory() 
      file:nil 
      types:nil] == NSOKButton)  {
    NSArray *filesToOpen = [oPanel filenames];
    NSString *nsFileName = [filesToOpen objectAtIndex:0];
    root_name=malloc(1024);
    
    [nsFileName 
        getFileSystemRepresentation:root_name
        maxLength:1024];
    printf("root_node=%s\n",root_name);
    
    root_node=explore_tree(root_name);
    if(!root_node) {
      free(root_name); root_name=NULL;
      NSRunAlertPanel(
          @"Shit !", 
          @"Could not explore dir...",
          @"Ok",nil,nil,nil);
      return;
    }
    [directoryNameDisplayer setStringValue:nsFileName];
    
    [self handleSizeChange];
    [self handleFontChange];
    [self setNeedsDisplay: YES];
  }

}

- (void) handleSizeChange {
  NSPoint pt_i=[self visibleRect].origin;
  
  [self setFrame:NSMakeRect(
      0,0,
      (root_node->depth+2.5)*x_scale,
      y_scale*root_node->size*512+10)];
  
  if(prev_y_scale!=0) {
    double o_i=(pt_i.y-5.0)/prev_y_scale;
    pt_i.y=5.0+o_i*y_scale;
    [self scrollPoint: pt_i];
  }  

  prev_y_scale=y_scale;
  
}

- (void) handleFontChange {
  NSRect br;
  font=[NSFont fontWithName:@"Monaco" size:10.0];
  font_asc=[font ascender];
  font_desc=[font descender];
  br=[font boundingRectForFont];
  font_off0=-br.origin.x;
  font_attrs=[[NSDictionary alloc] initWithObjectsAndKeys:
      font, NSFontAttributeName,
      nil];
}
static char *unit_names[]={
  "byte","kB","MB","GB"
};

static char *nameOfSize(double size) {
  static char buf[1024];
  int n;
  for(n=0;n<3;n++) {
    if(size<1024) break;
    size/=1024;
  }
  sprintf(buf,"%.3g %s",size,unit_names[n]);
  return buf;
}


static int dep;

- (void)recursiveDraw:(struct FileNode*)fn
    x_base:(double) x_base
    y_base:(double) y_base
    y_min: (double) y_min 
    y_max: (double) y_max {
  double height=fn->size*y_scale*512;
  double width=fn->flags & FN_STAT_DIR ?
    x_scale*.95 : x_scale*1.2;
  if(height==0 || y_base>y_max || y_base+height<y_min) 
    return;
  /*
  {
    int i;
    for(i=0;i<dep;i++) 
      printf("");
    printf(" %s\n",fn->name);
  }
  */
  if(height<1) {
    // Column is too short : just draw a line
/*
    [[NSColor blackColor] set];

    [NSBezierPath strokeRect:NSMakeRect(
        x_base,y_base,x_scale*(fn->depth+1),0)];
*/
    /*
    [NSBezierPath strokeLineFromPoint:
            NSMakePoint(x_base,y_base)
        toPoint:
            NSMakePoint(x_base+x_scale*(fn->depth+1),y_base)];
    */
  } else {
    { // draw rectangle
      double y0=MAX(y_base,y_min-10);
      double y1=MIN(y_base+height,y_max+10);
      double y_dr=y_base+fn->d_size*y_scale;
      double y_rp=y_dr+fn->r_size*y_scale;
      
      if(y_dr>y0) {
        [data_color set];
        [NSBezierPath fillRect: NSMakeRect(
            x_base,y0,width,MIN(y1,y_dr)-y0)];      
      }
      
      if(y_rp>y_dr+1 && !(y_rp<y0 || y_dr>y1)) {
        [rsrc_color set];
        [NSBezierPath fillRect: NSMakeRect(
            x_base,y_dr,width,y_rp-y_dr)];
      }
      
      if(y_base+height>y_rp+1 && !(y_base+height<y0 || y_rp>y1)) {
        [padding_color set];
        [NSBezierPath fillRect: NSMakeRect(
            x_base,y_rp,width,y_base+height-y_rp)];
      }
    
      [[NSColor blackColor] set];  
      [NSBezierPath strokeRect: NSMakeRect(
          x_base,y0,
          width,y1-y0)];
    }
    
    if(height>5) { // draw text
      double y0=MAX(y_base,y_min);
      double y1=MIN(y_base+height,y_max);
      NSRect text_clip;
      NSString *name=[NSString stringWithUTF8String:
          fn->name];
      NSString *text=[NSString localizedStringWithFormat:
          @"%@\n%s",name,nameOfSize(fn->size*512.0)];

      text_clip=NSMakeRect(
          x_base,y0,
          width,y1-y0);
      [text drawInRect:text_clip withAttributes: font_attrs];
    }
    if(fn->flags & FN_STAT_DIR && 
      !(fn->flags & FN_OPENDIR_ERROR)) { 
      struct FileNode *ch;
      dep++;
      for(ch=fn->children;ch;ch=ch->next) {
        [self recursiveDraw:ch
            x_base:x_base+x_scale
            y_base:y_base
            y_min:y_min
            y_max:y_max];
        y_base+=ch->size*512*y_scale;    
      }
      dep--;
    }    
  }  
}

- (void)drawRect:(NSRect)rect {
  
  [[NSColor whiteColor] set];
  NSRectFill(rect);
  
  if(!root_node) return;
  dep=0;
  [self recursiveDraw:root_node
      x_base:0.0
      y_base:5.0
      y_min :rect.origin.y
      y_max :rect.origin.y+rect.size.height];
}

- (void)setNeedsDisplayInRect:(NSRect)invalidRect {
  // No partial redraws
  [super setNeedsDisplayInRect:[self visibleRect]];
}


@end
