/*
 *  fileops.c
 *  CocoaDU
 *
 *  Created by Matthijs on Tue Jan 06 1970.
 *  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h> 
#include <errno.h> 

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>

#include "fileops.h"

typedef struct FileNode FileNode;

/*
static double updater_interval,
static void (*updater)(int n,FileNode **));
*/
static void fillin_fields(
    char *name_buffer,
    FileNode *fn) {
  int ret;
  fn->st=malloc(sizeof(struct stat));

  ret=lstat(name_buffer,fn->st);
  if(ret==-1) {
    fprintf(stderr,"stat(\"%s\"): %s\n",
        name_buffer,strerror(errno));
    fn->flags=FN_STAT_ERROR;
    fn->size=0;
    free(fn->st);
    fn->st=(void*)errno;
    return;
  }
    
  fn->flags=0;
  fn->size=fn->st->st_blocks;
  fn->d_size=fn->st->st_size;
  if(fn->st->st_mode & S_IFDIR)
    fn->flags|=FN_STAT_DIR;
  if(fn->st->st_mode & S_IFLNK) 
    fn->flags|=FN_STAT_SYMLINK;  
  if(fn->st->st_mode & 
      (S_IFCHR | S_IFSOCK | S_IFIFO)) 
    fn->flags|=FN_STAT_OTHER;

  { /* handle ressource fork */
    struct stat st;
    char *end_p;
    for(end_p=name_buffer;*end_p;end_p++);
    strcpy(end_p,"/..namedfork/rsrc");
    ret=lstat(name_buffer,&st);
    if(ret==-1) {
      fn->flags|=FN_STAT_RSRC_ERROR;
      fn->r_size=0;    
    } else {
      fn->r_size=st.st_size;
    }
    *end_p=0;
  }
  
}

#define MAX(a,b) ((a)>(b) ? (a) : (b))

static void explore_dir(
    char *name_buffer,
    FileNode *root_node
    ) {
  DIR *dir;
  struct dirent *de;
  int ret;
  char *end_p;
  FileNode **fn_w=&root_node->children;
  unsigned long tot_size;
  int max_depth;
  
  printf("-> %s\n",name_buffer);
  
  for(end_p=name_buffer;*end_p;end_p++);
  *end_p++='/';
  *end_p=0;
  
  dir=opendir(name_buffer);
  if(!dir) {
    root_node->flags|=FN_OPENDIR_ERROR;
    root_node->children=(void*)errno; 
    return;
  }
  
  tot_size=root_node->size;
  max_depth=0;
  
  while((de=readdir(dir))) {
    FileNode *fn;
    
    if(!strcmp(de->d_name,".") ||
       !strcmp(de->d_name,"..")) continue;
/*
    printf("---> %s\n",de->d_name);
  */  
    fn=malloc(sizeof(FileNode)+strlen(de->d_name));
    strcpy(fn->name,de->d_name);

    *fn_w=fn;
    fn_w=&fn->next;
    
    strcpy(end_p,de->d_name);

    fillin_fields(name_buffer,fn);

    if(fn->flags & FN_STAT_ERROR) {
      goto bail;
      fn->size=-1;
    }
    
    if(fn->flags & FN_STAT_DIR) {
      explore_dir(name_buffer,fn);
      tot_size+=fn->size;
      max_depth=MAX(max_depth,fn->depth+1);     
    } else {
      tot_size+=fn->size;      
    }
    
  bail:
    *end_p=0;
  }
  *fn_w=NULL;
  ret=closedir(dir);
  
  root_node->size=tot_size;
  root_node->depth=max_depth;
}

FileNode *explore_tree(
    char *root) {
  FileNode *root_node=NULL;
  char name_buffer[MAXNAMLEN+1];
  char *name;
  
  strcpy(name_buffer,root);

  { 
    char *p;
    for(p=name_buffer;*p;p++);
    if(p==name_buffer) {
      fprintf(stderr,"empty dir name\n");
      goto bail;
    }
    p--;
    while(p>=name_buffer && *p=='/') p--;

    if(p<name_buffer) {
      name=name_buffer; /* ="/" */
      name[1]=0;
    } else {
      p[1]=0;
      while(p>=name_buffer && *p!='/') p--;
      name=*p=='/' ? p+1 : p;
    }
  }  
  
  root_node=malloc(sizeof(FileNode)+strlen(name));
  strcpy(root_node->name,name);
  
  fillin_fields(name_buffer,root_node);
  if(root_node->flags & FN_STAT_ERROR) {
    fprintf(stderr,"stat \"%s\": %s\n",
      name_buffer,strerror((int)root_node->st));
    goto bail;
  } 
  if(!(root_node->flags & FN_STAT_DIR)) {
    fprintf(stderr," file \"%s\" is not a directory\n",
      name_buffer);
    goto bail;
  } 
  explore_dir(name_buffer,root_node);
  root_node->next=NULL;

  return root_node;
  
bail:

  if(root_node) free(root_node);
  return NULL;
}

void set_unknown_sizes(FileNode *fn,unsigned long us) {
  

}

void dealloc_tree(FileNode *fn) {

}

